/*
 *  Licensed to the author under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.java.quickcheck;

import static java.lang.Boolean.*;

import java.io.PrintWriter;

public class RunnerFactory {

	public Runner<Object> createRunner(Generator<Object> generator, int runs, Characteristic<Object> characteristic,
			boolean verbose) {
		PrintWriter writer = verbose ? new PrintWriter(System.out) : new PrintWriter(new NullWriter());
		return createRunnerInstance(characteristic, runs, generator, writer);
	}

	public <T> Runner<T> createRunnerInstance(Characteristic<T> characteristic, int runs, Generator<T> generator,
			PrintWriter writer) {
		Runner<T> runner = createSimpleRunner(characteristic, runs, generator, writer);
		if(getBoolean(QuickCheck.SYSTEM_PROPERTY_SERIALIZE)) {
			return new SerializingRunnerDecorator<T>(runner,
					getBoolean(QuickCheck.SYSTEM_PROPERTY_SERIALIZE_DELETE_OLD));
		}
		if(getBoolean(QuickCheck.SYSTEM_PROPERTY_DESERIALIZE)) {
			return new DeserializingRunnerDecorator<T>(runner);
		}
		return runner;
	}

	<T> Runner<T> createSimpleRunner(Characteristic<T> characteristic, int runs, Generator<T> generator,
			PrintWriter writer) {
		RunnerImpl<T> runnerImpl = new RunnerImpl<T>();
		runnerImpl.setCharacteristic(characteristic);
		runnerImpl.setRuns(runs);
		runnerImpl.setGenerator(generator);
		runnerImpl.setWriter(writer);
		return runnerImpl;
	}
}