/*
 *  Licensed to the author under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.java.quickcheck.generator.support;

import java.util.HashSet;
import java.util.Set;

import net.java.quickcheck.Generator;
import net.java.quickcheck.StatefulGenerator;

public class UniqueValuesGenerator<T> extends VetoableGenerator<T> implements
		StatefulGenerator<T> {

	private final Set<T> fValues = new HashSet<T>();

	public UniqueValuesGenerator(Generator<T> generator) {
		this(generator, DEFAULT_MAX_TRIES);
	}

	public UniqueValuesGenerator(Generator<T> generator, int defaultMaxTries) {
		super(generator, defaultMaxTries);
		reset();
	}

	@Override
	protected boolean tryValue(T value) {
		return fValues.add(value);
	}

	public void reset() {
		fValues.clear();
	}
}
