package net.java.quickcheck.junit;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

import net.java.quickcheck.Generator;
import net.java.quickcheck.QuickCheck;

/**
 * The ForAll annotation is used to add quickcheck specific metadata to test
 * classes.
 */
@Retention(RetentionPolicy.RUNTIME)
public @interface ForAll {

	String NO_GENERATOR = "__NO_GENERATOR__";

	interface NoGenerator<T> extends Generator<Object> {
	}

	/**
	 * Number of executions per test method.
	 * 
	 */
	int runs() default QuickCheck.MAX_NUMBER_OF_RUNS;

	/**
	 * List of methods returning a generator instance. The methods can be public
	 * instance methods of the test class or full qualified static test methods
	 * (package.Class.method).
	 */
	String[] generatorMethod() default NO_GENERATOR;

	/**
	 * Enabled and disable output.
	 */
	boolean verbose() default false;

	/**
	 * Specify a generator class directly (without using a generatorMethod
	 * annotation).
	 */
	Class<?>[] generatorClass() default NoGenerator.class;
}
